@@ -1,5 +1,6 @@
 /* This file is part of the KDE libraries
    Copyright (C) 2005-2006 Olivier Goffart <ogoffart at kde.org>
+   Copyright (C) 2013-2014 Martin Klapetek <mklapetek@kde.org>
 
    code from KNotify/KNotifyClient
    Copyright (c) 1997 Christian Esken (esken@kde.org)
@@ -49,6 +50,7 @@ struct KNotification::Private {
     QWidget *widget;
     QString title;
     QString text;
+    QString iconName;
     QStringList actions;
     QPixmap pixmap;
     ContextList contexts;
@@ -90,6 +92,7 @@ KNotification::KNotification(
     connect(&d->updateTimer, SIGNAL(timeout()), this, SLOT(update()));
     d->updateTimer.setSingleShot(true);
     d->updateTimer.setInterval(100);
+    d->widget = nullptr;
 }
 
 KNotification::~KNotification()
@@ -123,7 +126,7 @@ QWidget *KNotification::widget() const
 void KNotification::setWidget(QWidget *wid)
 {
     d->widget = wid;
-    setParent(wid);
+//     setParent(wid);
     if (wid && d->flags &  CloseWhenWidgetActivated) {
         wid->installEventFilter(this);
     }
@@ -131,6 +134,10 @@ void KNotification::setWidget(QWidget *wid)
 
 void KNotification::setTitle(const QString &title)
 {
+    if (title == d->title) {
+        return;
+    }
+
     d->needUpdate = true;
     d->title = title;
     if (d->id > 0) {
@@ -140,6 +147,10 @@ void KNotification::setTitle(const QString &title)
 
 void KNotification::setText(const QString &text)
 {
+    if (text == d->text) {
+        return;
+    }
+
     d->needUpdate = true;
     d->text = text;
     if (d->id > 0) {
@@ -147,6 +158,24 @@ void KNotification::setText(const QString &text)
     }
 }
 
+void KNotification::setIconName(const QString &icon)
+{
+    if (icon == d->iconName) {
+        return;
+    }
+
+    d->needUpdate = true;
+    d->iconName = icon;
+    if (d->id > 0) {
+        d->updateTimer.start();
+    }
+}
+
+QString KNotification::iconName() const
+{
+    return d->iconName;
+}
+
 QPixmap KNotification::pixmap() const
 {
     return d->pixmap;
@@ -168,6 +197,10 @@ QStringList KNotification::actions() const
 
 void KNotification::setActions(const QStringList &as)
 {
+    if (as == d->actions) {
+        return;
+    }
+
     d->needUpdate = true;
     d->actions = as;
     if (d->id > 0) {
@@ -301,23 +334,8 @@ KNotification *KNotification::event(const QString &eventid, const QString &text,
 KNotification *KNotification::event(StandardEvent eventid, const QString &title, const QString &text,
                                     const QPixmap &pixmap, QWidget *widget, const NotificationFlags &flags)
 {
-    QString message;
-    switch (eventid) {
-    case Warning:
-        message = QLatin1String("warning");
-        break;
-    case Error:
-        message = QLatin1String("fatalerror");
-        break;
-    case Catastrophe:
-        message = QLatin1String("catastrophe");
-        break;
-    case Notification: // fall through
-    default:
-        message = QLatin1String("notification");
-        break;
-    }
-    return event(message, title, text, pixmap, widget, flags | DefaultEvent);
+
+    return event(standardEventToEventId(eventid), title, text, pixmap, widget, flags | DefaultEvent);
 }
 
 KNotification *KNotification::event(StandardEvent eventid, const QString &text,
@@ -326,6 +344,34 @@ KNotification *KNotification::event(StandardEvent eventid, const QString &text,
     return event(eventid, QString(), text, pixmap, widget, flags);
 }
 
+KNotification *KNotification::event(const QString &eventid, const QString &title, const QString &text,
+                                const QString &iconName, QWidget *widget,
+                                const NotificationFlags &flags, const QString &componentName)
+{
+    KNotification *notify = new KNotification(eventid, widget, flags);
+    notify->setTitle(title);
+    notify->setText(text);
+    notify->setIconName(iconName);
+    notify->setComponentName(flags & DefaultEvent ? QStringLiteral("plasma_workspace") : componentName);
+
+    QTimer::singleShot(0, notify, SLOT(sendEvent()));
+
+    return notify;
+}
+
+KNotification *KNotification::event(StandardEvent eventid, const QString &title, const QString &text,
+                                    const QString &iconName, QWidget *widget,
+                                    const NotificationFlags &flags)
+{
+    return event(standardEventToEventId(eventid), title, text, iconName, widget, flags | DefaultEvent);
+}
+
+KNotification* KNotification::event(StandardEvent eventid, const QString &title, const QString &text,
+                                    QWidget *widget, const NotificationFlags &flags)
+{
+    return event(standardEventToEventId(eventid), title, text, standardEventToIconName(eventid), widget, flags | DefaultEvent);
+}
+
 void KNotification::ref()
 {
     d->ref++;
@@ -342,7 +388,7 @@ void KNotification::deref()
 
 void KNotification::beep(const QString &reason, QWidget *widget)
 {
-    event(QLatin1String("beep"), reason, QPixmap(), widget, CloseOnTimeout | DefaultEvent);
+    event(QStringLiteral("beep"), reason, QPixmap(), widget, CloseOnTimeout | DefaultEvent);
 }
 
 void KNotification::sendEvent()
@@ -357,6 +403,9 @@ void KNotification::sendEvent()
 
 int KNotification::id()
 {
+    if (!d) {
+        return -1;
+    }
     return d->id;
 }
 
@@ -365,7 +414,7 @@ QString KNotification::appName() const
     QString appname;
 
     if (d->flags & DefaultEvent) {
-        appname = QLatin1String("plasma_workspace");
+        appname = QStringLiteral("plasma_workspace");
     } else if (!d->componentName.isEmpty()) {
         appname = d->componentName;
     } else {
@@ -377,7 +426,9 @@ QString KNotification::appName() const
 
 void KNotification::update()
 {
-    KNotificationManager::self()->update(this);
+    if (d->needUpdate) {
+        KNotificationManager::self()->update(this);
+    }
 }
 
 bool KNotification::eventFilter(QObject *watched, QEvent *event)
@@ -393,3 +444,46 @@ bool KNotification::eventFilter(QObject *watched, QEvent *event)
     return false;
 }
 
+QString KNotification::standardEventToEventId(KNotification::StandardEvent event)
+{
+    QString eventId;
+    switch (event) {
+        case Warning:
+            eventId = QStringLiteral("warning");
+            break;
+        case Error:
+            eventId = QStringLiteral("fatalerror");
+            break;
+        case Catastrophe:
+            eventId = QStringLiteral("catastrophe");
+            break;
+        case Notification: // fall through
+        default:
+            eventId = QStringLiteral("notification");
+            break;
+    }
+    return eventId;
+}
+
+
+QString KNotification::standardEventToIconName(KNotification::StandardEvent event)
+{
+    QString iconName;
+    switch (event) {
+        case Warning:
+            iconName = QStringLiteral("dialog-warning");
+            break;
+        case Error:
+            iconName = QStringLiteral("dialog-error");
+            break;
+        case Catastrophe:
+            iconName = QStringLiteral("dialog-error");
+            break;
+        case Notification: // fall through
+        default:
+            iconName = QStringLiteral("dialog-information");
+            break;
+    }
+    return iconName;
+}
+
